@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-2001  The Stanford SRP Authentication Project
+ * Copyright (c) 1997-2003  The Stanford SRP Authentication Project
  * All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
@@ -38,6 +38,7 @@
  * 3. Redistributions in source or binary form must retain an intact copy
  *    of this copyright notice and list of conditions.
  */
+#include "t_defines.h"
 #include "srp.h"
 #include "t_sha.h"
 
@@ -60,7 +61,7 @@ struct server_meth_st {
 
 #define SERVER_CTXP(srp)    ((struct server_meth_st *)(srp)->meth_data)
 
-static int
+static SRP_RESULT
 srp2945_server_init(SRP * srp)
 {
   srp->magic = SRP_MAGIC_SERVER;
@@ -72,7 +73,7 @@ srp2945_server_init(SRP * srp)
   return SRP_SUCCESS;
 }
 
-static int
+static SRP_RESULT
 srp2945_server_finish(SRP * srp)
 {
   if(srp->meth_data) {
@@ -82,17 +83,7 @@ srp2945_server_finish(SRP * srp)
   return SRP_SUCCESS;
 }
 
-static int
-BigIntegerToCstr(BigInteger x, cstr * out)
-{
-  if(cstr_set_length(out, (BigIntegerBitLen(x) + 7) / 8) < 0)
-    return SRP_ERROR;
-  if(cstr_set_length(out, BigIntegerToBytes(x, out->data)) < 0)
-    return SRP_ERROR;
-  return SRP_SUCCESS;
-}
-
-static int
+static SRP_RESULT
 srp2945_server_params(SRP * srp, const unsigned char * modulus, int modlen,
 		      const unsigned char * generator, int genlen,
 		      const unsigned char * salt, int saltlen)
@@ -101,15 +92,7 @@ srp2945_server_params(SRP * srp, const unsigned char * modulus, int modlen,
   SHA1_CTX ctxt;
   int i;
 
-  if(modulus == NULL || generator == NULL || salt == NULL)
-    return SRP_ERROR;
-
-  /* Set fields in SRP context */
-  srp->modulus = BigIntegerFromBytes(modulus, modlen);
-  srp->generator = BigIntegerFromBytes(generator, genlen);
-  if(srp->salt == NULL)
-    srp->salt = cstr_new();
-  cstr_setn(srp->salt, salt, saltlen);
+  /* Fields set by SRP_set_params */
 
   /* Update hash state */
   SHA1Init(&ctxt);
@@ -139,7 +122,7 @@ srp2945_server_params(SRP * srp, const unsigned char * modulus, int modlen,
   return SRP_SUCCESS;
 }
 
-static int
+static SRP_RESULT
 srp2945_server_auth(SRP * srp, const unsigned char * a, int alen)
 {
   /* On the server, the authenticator is the verifier */
@@ -149,7 +132,7 @@ srp2945_server_auth(SRP * srp, const unsigned char * a, int alen)
 }
 
 /* Normally this method isn't called, except maybe by test programs */
-static int
+static SRP_RESULT
 srp2945_server_passwd(SRP * srp, const unsigned char * p, int plen)
 {
   SHA1_CTX ctxt;
@@ -172,20 +155,19 @@ srp2945_server_passwd(SRP * srp, const unsigned char * p, int plen)
 
   /* verifier = g^x mod N */
   srp->verifier = BigIntegerFromInt(0);
-  BigIntegerModExp(srp->verifier, srp->generator, srp->password, srp->modulus);
+  BigIntegerModExp(srp->verifier, srp->generator, srp->password, srp->modulus, srp->bctx, srp->accel);
 
   return SRP_SUCCESS;
 }
 
-static int
+static SRP_RESULT
 srp2945_server_genpub(SRP * srp, cstr ** result)
 {
   cstr * bstr;
   SHA1_CTX ctxt;
   unsigned char dig[SHA_DIGESTSIZE];
-  unsigned char sbuf[SRP_SECRET_MAXBYTES];
   unsigned long u;
-  int slen = (SRP_SECRET_BITS(BigIntegerBitLen(srp->modulus)) + 7) / 8;
+  int slen = (SRP_get_secret_bits(BigIntegerBitLen(srp->modulus)) + 7) / 8;
 
   if(result == NULL)
     bstr = cstr_new();
@@ -195,14 +177,15 @@ srp2945_server_genpub(SRP * srp, cstr ** result)
     bstr = *result;
   }
 
-  t_random(sbuf, slen);
-  srp->secret = BigIntegerFromBytes(sbuf, slen);
+  cstr_set_length(bstr, BigIntegerByteLen(srp->modulus));
+  t_random(bstr->data, slen);
+  srp->secret = BigIntegerFromBytes(bstr->data, slen);
   srp->pubkey = BigIntegerFromInt(0);
   u = 0;
   /* Screen for u != 0 */
   while(u == 0) {
     /* B = v + g^b mod n (blinding) */
-    BigIntegerModExp(srp->pubkey, srp->generator, srp->secret, srp->modulus);
+    BigIntegerModExp(srp->pubkey, srp->generator, srp->secret, srp->modulus, srp->bctx, srp->accel);
     BigIntegerAdd(srp->pubkey, srp->pubkey, srp->verifier);
     if(BigIntegerCmp(srp->pubkey, srp->modulus) > 0)
       BigIntegerSub(srp->pubkey, srp->pubkey, srp->modulus);
@@ -214,7 +197,7 @@ srp2945_server_genpub(SRP * srp, cstr ** result)
     SHA1Final(dig, &ctxt);	/* dig = H(pubkey) */
     u = (dig[0] << 24) | (dig[1] << 16) | (dig[2] << 8) | dig[3];
   }
-  srp->u = BigIntegerFromInt(u);
+  srp->u = BigIntegerFromBytes(dig, 4);
 
   /* oldckhash: B */
   SHA1Update(&SERVER_CTXP(srp)->oldckhash, bstr->data, bstr->length);
@@ -225,12 +208,11 @@ srp2945_server_genpub(SRP * srp, cstr ** result)
   return SRP_SUCCESS;
 }
 
-static int
+static SRP_RESULT
 srp2945_server_key(SRP * srp, cstr ** result,
 		   const unsigned char * pubkey, int pubkeylen)
 {
   cstr * s;
-  cstr * r;
   BigInteger t1, t2, t3;
 
   /* ckhash: (H(N) xor H(g)) | H(U) | s | A */
@@ -249,10 +231,10 @@ srp2945_server_key(SRP * srp, cstr ** result,
 
   /* compute A*v^u */
   t1 = BigIntegerFromInt(0);
-  BigIntegerModExp(t1, srp->verifier, srp->u, srp->modulus); /* t1 = v^u */
+  BigIntegerModExp(t1, srp->verifier, srp->u, srp->modulus, srp->bctx, srp->accel); /* t1 = v^u */
   t2 = BigIntegerFromBytes(pubkey, pubkeylen); /* t2 = A */
   t3 = BigIntegerFromInt(0);
-  BigIntegerModMul(t3, t2, t1, srp->modulus);	/* t3 = A*v^u (mod N) */
+  BigIntegerModMul(t3, t2, t1, srp->modulus, srp->bctx); /* t3 = A*v^u (mod N) */
 
   if(BigIntegerCmpInt(t3, 1) <= 0) {	/* Reject A*v^u == 0,1 (mod N) */
     BigIntegerFree(t1);
@@ -272,7 +254,7 @@ srp2945_server_key(SRP * srp, cstr ** result,
   }
 
   srp->key = BigIntegerFromInt(0);
-  BigIntegerModExp(srp->key, t3, srp->secret, srp->modulus);  /* (Av^u)^b */
+  BigIntegerModExp(srp->key, t3, srp->secret, srp->modulus, srp->bctx, srp->accel);  /* (Av^u)^b */
   BigIntegerFree(t1);
   BigIntegerFree(t2);
   BigIntegerFree(t3);
@@ -304,7 +286,7 @@ srp2945_server_key(SRP * srp, cstr ** result,
   return SRP_SUCCESS;
 }
 
-static int
+static SRP_RESULT
 srp2945_server_verify(SRP * srp, const unsigned char * proof, int prooflen)
 {
   unsigned char expected[SHA_DIGESTSIZE];
@@ -325,7 +307,7 @@ srp2945_server_verify(SRP * srp, const unsigned char * proof, int prooflen)
   return SRP_ERROR;
 }
 
-static int
+static SRP_RESULT
 srp2945_server_respond(SRP * srp, cstr ** proof)
 {
   if(proof == NULL)
