@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-2001  The Stanford SRP Authentication Project
+ * Copyright (c) 1997-2002  The Stanford SRP Authentication Project
  * All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
@@ -42,6 +42,7 @@
 /*#define _POSIX_SOURCE*/
 #include <stdio.h>
 #include "t_defines.h"
+#include "cstr.h"
 
 static int
 hexDigitToInt(c)
@@ -62,7 +63,8 @@ hexDigitToInt(c)
  */
 _TYPE( int )
 t_fromhex(dst, src)
-     register char *dst, *src;
+     char * dst;
+     const char * src;
 {
   register char *chp = dst;
   register unsigned size = strlen(src);
@@ -87,8 +89,9 @@ t_fromhex(dst, src)
  */
 _TYPE( char * )
 t_tohex(dst, src, size)
-     register char *dst, *src;
-     register unsigned size;
+     char * dst;
+     const char * src;
+     unsigned size;
 {
    int notleading = 0;
 
@@ -111,6 +114,16 @@ t_tohex(dst, src, size)
    return dst;
 }
 
+_TYPE( char * )
+t_tohexcstr(dst, src, size)
+     cstr * dst;
+     const char * src;
+     unsigned size;
+{
+  cstr_set_length(dst, 2 * size + 1);
+  return t_tohex(dst->data, src, size);
+}
+
 static char b64table[] =
   "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz./";
 
@@ -119,7 +132,8 @@ static char b64table[] =
  */
 _TYPE( int )
 t_fromb64(dst, src)
-     register char *dst, *src;
+     char * dst;
+     const char * src;
 {
   unsigned char *a;
   char *loc;
@@ -176,13 +190,26 @@ t_fromb64(dst, src)
   return size - j + 1;
 }
 
+_TYPE( int )
+t_cstrfromb64(dst, src)
+     cstr * dst;
+     const char * src;
+{
+  int len;
+  cstr_set_length(dst, (strlen(src) * 6 + 7) / 8);
+  len = t_fromb64(dst->data, src);
+  cstr_set_length(dst, len);
+  return len;
+}
+
 /*
  * Convert a raw byte string into a null-terminated base64 ASCII string.
  */
 _TYPE( char * )
 t_tob64(dst, src, size)
-     register char *dst, *src;
-     register unsigned size;
+     char * dst;
+     const char * src;
+     unsigned size;
 {
   int c, pos = size % 3;
   unsigned char b0 = 0, b1 = 0, b2 = 0, notleading = 0;
@@ -231,3 +258,13 @@ t_tob64(dst, src, size)
   *dst++ = '\0';
   return olddst;
 }
+
+_TYPE( char * )
+t_tob64cstr(dst, src, sz)
+     cstr * dst;
+     const char * src;
+     unsigned int sz;
+{
+  cstr_set_length(dst, (sz * 8 + 5) / 6 + 1);
+  return t_tob64(dst->data, src, sz);
+}
