@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1999  The Stanford SRP Authentication Project
+ * Copyright (c) 1997-2003  The Stanford SRP Authentication Project
  * All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
@@ -26,14 +26,14 @@
  * In addition, the following conditions apply:
  *
  * 1. Any software that incorporates the SRP authentication technology
- *    must display the following acknowlegment:
+ *    is requested to display the following acknowlegment:
  *    "This product uses the 'Secure Remote Password' cryptographic
  *     authentication system developed by Tom Wu (tjw@CS.Stanford.EDU)."
  *
  * 2. Any software that incorporates all or part of the SRP distribution
- *    itself must also display the following acknowledgment:
+ *    itself must display the following acknowledgment:
  *    "This product includes software developed by Tom Wu and Eugene
- *     Jhong for the SRP Distribution (http://srp.stanford.edu/srp/)."
+ *     Jhong for the SRP Distribution (http://srp.stanford.edu/)."
  *
  * 3. Redistributions in source or binary form must retain an intact copy
  *    of this copyright notice and list of conditions.
@@ -137,48 +137,6 @@ char *strchr(), *strrchr(), *strtok();
 #include <sys/timeb.h>
 #endif
 
-/* Should be defined by configure; GNU_MP is default */
-/* #define GNU_MP 1 */
-/* #define OPENSSL 1 */
-/* #define CRYPTOLIB 1 */
-
-#ifndef MATH_PRIV
-typedef void * BigInteger;
-#endif
-
-_TYPE( BigInteger ) BigIntegerFromInt P((unsigned int number));
-_TYPE( BigInteger ) BigIntegerFromBytes P((const unsigned char * bytes,
-					   int length));
-_TYPE( int ) BigIntegerToBytes P((BigInteger src, unsigned char * dest));
-_TYPE( void ) BigIntegerToHex P((BigInteger src, char * dest));
-_TYPE( void ) BigIntegerToString P((BigInteger src, char * dest,
-				    unsigned int radix));
-_TYPE( int ) BigIntegerBitLen P((BigInteger b));
-_TYPE( int ) BigIntegerCmp P((BigInteger c1, BigInteger c2));
-_TYPE( int ) BigIntegerCmpInt P((BigInteger c1, unsigned int c2));
-_TYPE( void ) BigIntegerLShift P((BigInteger result, BigInteger x,
-				unsigned int bits));
-_TYPE( void ) BigIntegerAdd P((BigInteger result, BigInteger a1, BigInteger a2));
-_TYPE( void ) BigIntegerAddInt P((BigInteger result,
-				BigInteger a1, unsigned int a2));
-_TYPE( void ) BigIntegerSub P((BigInteger result, BigInteger s1, BigInteger s2));
-_TYPE( void ) BigIntegerSubInt P((BigInteger result,
-				BigInteger s1, unsigned int s2));
-/* For BigIntegerMul{,Int}: result != m1, m2 */
-_TYPE( void ) BigIntegerMul P((BigInteger result, BigInteger m1, BigInteger m2));
-_TYPE( void ) BigIntegerMulInt P((BigInteger result,
-				BigInteger m1, unsigned int m2));
-_TYPE( void ) BigIntegerDivInt P((BigInteger result,
-				BigInteger d, unsigned int m));
-_TYPE( void ) BigIntegerMod P((BigInteger result, BigInteger d, BigInteger m));
-_TYPE( unsigned int ) BigIntegerModInt P((BigInteger d, unsigned int m));
-_TYPE( void ) BigIntegerModMul P((BigInteger result,
-				BigInteger m1, BigInteger m2, BigInteger m));
-_TYPE( void ) BigIntegerModExp P((BigInteger result, BigInteger base,
-				BigInteger expt, BigInteger modulus));
-_TYPE( void ) BigIntegerModExpInt P((BigInteger result, BigInteger base,
-				   unsigned int expt, BigInteger modulus));
-_TYPE( int ) BigIntegerCheckPrime P((BigInteger n));
-_TYPE( void ) BigIntegerFree P((BigInteger b));
+/* Looking for BigInteger math functions?  They've moved to <srp_aux.h>. */
 
 #endif
