@@ -15,6 +15,13 @@
 #define OPENSSL_SHA 1
 #endif
 
+#ifdef TOMCRYPT
+# include <mycrypt.h>
+# ifdef SHA1
+#  define TOMCRYPT_SHA 1
+# endif
+#endif
+
 #ifdef CRYPTOLIB
 /* The SHA (shs) implementation in CryptoLib 1.x breaks when Update
  * is called multiple times, so we still use our own code.
@@ -29,6 +36,13 @@ typedef SHA_CTX SHA1_CTX;
 #define SHA1Init SHA1_Init
 #define SHA1Update SHA1_Update
 #define SHA1Final SHA1_Final
+#elif defined(TOMCRYPT_SHA)
+/* mycrypt.h already included above */
+
+typedef hash_state SHA1_CTX;
+#define SHA1Init sha1_init
+#define SHA1Update sha1_process
+#define SHA1Final(D,C) sha1_done(C,D)
 #elif defined(CRYPTOLIB_SHA)
 #include "libcrypt.h"
 
