@@ -545,7 +545,7 @@ cast_fb64_session(key, server, fbp, fs)
 	if(key->length >= 2 * klen + sizeof(Block))
 	  memmove(fbp->temp_feed, key->data + 2 * klen, sizeof(Block));
 	else
-#ifdef SRP
+#ifdef HAVE_SRP
           t_random(fbp->temp_feed, sizeof(Block));
 #else
           memset(fbp->temp_feed, 0, sizeof(Block));
