@@ -55,6 +55,10 @@ static char sccsid[] = "@(#)encrypt.c	8.2 (Berkeley) 5/30/95";
  * or implied warranty.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #ifdef	ENCRYPTION
 
 #define	ENCRYPT_NAMES
@@ -66,12 +70,16 @@ static char sccsid[] = "@(#)encrypt.c	8.2 (Berkeley) 5/30/95";
 #ifdef	__STDC__
 #include <stdlib.h>
 #endif
-#ifdef	NO_STRING_H
-#include <strings.h>
-#else
+#ifdef	HAVE_STRING_H
 #include <string.h>
+#else
+#include <strings.h>
 #endif
 
+#ifdef TLS
+extern int	tls_active;
+#endif /* TLS */
+
 /*
  * These functions pointers point to the current routines
  * for encrypting and decrypting data.
@@ -96,10 +104,8 @@ static char *Name = "Noname";
 
 #define	typemask(x)	((x) > 0 ? 1 << ((x)-1) : 0)
 
-static long i_support_encrypt = typemask(ENCTYPE_DES_CFB64)
-				| typemask(ENCTYPE_DES_OFB64);
-static long i_support_decrypt = typemask(ENCTYPE_DES_CFB64)
-				| typemask(ENCTYPE_DES_OFB64);
+static long i_support_encrypt = 0;
+static long i_support_decrypt = 0;
 static long i_wont_support_encrypt = 0;
 static long i_wont_support_decrypt = 0;
 #define	I_SUPPORT_ENCRYPT	(i_support_encrypt & ~i_wont_support_encrypt)
@@ -110,32 +116,33 @@ static long remote_supports_decrypt = 0;
 
 static Encryptions encryptions[] = {
 #ifdef	DES_ENCRYPTION
-    { "DES3_CFB64",
-          ENCTYPE_DES3_CFB64,
-          des3_cfb64_encrypt,	
-          des3_cfb64_decrypt,
-          des3_cfb64_init,
-          des3_cfb64_start,
-          des3_cfb64_is,
-          des3_cfb64_reply,
-          des3_cfb64_session,
-          des3_cfb64_keyid,
-          des3_cfb64_printsub },
-    { "DES3_OFB64",
-          ENCTYPE_DES3_OFB64,
-          des3_ofb64_encrypt,	
-          des3_ofb64_decrypt,
-          des3_ofb64_init,
-          des3_ofb64_start,
-          des3_ofb64_is,
-          des3_ofb64_reply,
-          des3_ofb64_session,
-          des3_ofb64_keyid,
-          des3_ofb64_printsub },
-#endif 
+  { "DES3_CFB64",	ENCTYPE_DES3_CFB64,
+			ENCFLAG_CLIENT_ENABLE,
+			des3_cfb64_encrypt,
+			des3_cfb64_decrypt,
+			des3_cfb64_init,
+			des3_cfb64_start,
+			des3_cfb64_is,
+			des3_cfb64_reply,
+			des3_cfb64_session,
+			des3_cfb64_keyid,
+			des3_cfb64_printsub },
+  { "DES3_OFB64",	ENCTYPE_DES3_OFB64,
+			ENCFLAG_DEFAULT,
+			des3_ofb64_encrypt,
+			des3_ofb64_decrypt,
+			des3_ofb64_init,
+			des3_ofb64_start,
+			des3_ofb64_is,
+			des3_ofb64_reply,
+			des3_ofb64_session,
+			des3_ofb64_keyid,
+			des3_ofb64_printsub },
+#endif /* DES_ENCRYPTION */
 #ifdef CAST_ENCRYPTION
 # ifndef CAST_EXPORT_ENCRYPTION
-    { "CAST128_CFB64",	ENCTYPE_CAST128_CFB64,
+  { "CAST128_CFB64",	ENCTYPE_CAST128_CFB64,
+			ENCFLAG_CLIENT_ENABLE,
 			cast_cfb64_encrypt,
 			cast_cfb64_decrypt,
 			cast_cfb64_init,
@@ -145,7 +152,8 @@ static Encryptions encryptions[] = {
 			cast_cfb64_session,
 			cast_cfb64_keyid,
 			cast_cfb64_printsub },
-    { "CAST128_OFB64",	ENCTYPE_CAST128_OFB64,
+  { "CAST128_OFB64",	ENCTYPE_CAST128_OFB64,
+			ENCFLAG_DEFAULT,
 			cast_ofb64_encrypt,
 			cast_ofb64_decrypt,
 			cast_ofb64_init,
@@ -155,10 +163,11 @@ static Encryptions encryptions[] = {
 			cast_ofb64_session,
 			cast_ofb64_keyid,
 			cast_ofb64_printsub },
-# endif
-#endif
+# endif /* CAST_EXPORT_ENCRYPTION */
+#endif /* CAST_ENCRYPTION */
 #ifdef	DES_ENCRYPTION
-    { "DES_CFB64",	ENCTYPE_DES_CFB64,
+  { "DES_CFB64",	ENCTYPE_DES_CFB64,
+			ENCFLAG_CLIENT_ENABLE,
 			cfb64_encrypt,
 			cfb64_decrypt,
 			cfb64_init,
@@ -168,7 +177,8 @@ static Encryptions encryptions[] = {
 			cfb64_session,
 			cfb64_keyid,
 			cfb64_printsub },
-    { "DES_OFB64",	ENCTYPE_DES_OFB64,
+  { "DES_OFB64",	ENCTYPE_DES_OFB64,
+			ENCFLAG_DEFAULT,
 			ofb64_encrypt,
 			ofb64_decrypt,
 			ofb64_init,
@@ -180,7 +190,8 @@ static Encryptions encryptions[] = {
 			ofb64_printsub },
 #endif	/* DES_ENCRYPTION */
 #if defined(CAST_EXPORT_ENCRYPTION) || defined(CAST_ENCRYPTION)
-    { "CAST5_40_CFB64",	ENCTYPE_CAST5_40_CFB64,
+  { "CAST5_40_CFB64",	ENCTYPE_CAST5_40_CFB64,
+			ENCFLAG_CLIENT_ENABLE,
 			castexp_cfb64_encrypt,
 			castexp_cfb64_decrypt,
 			castexp_cfb64_init,
@@ -190,7 +201,8 @@ static Encryptions encryptions[] = {
 			castexp_cfb64_session,
 			castexp_cfb64_keyid,
 			cast_cfb64_printsub },
-    { "CAST5_40_OFB64",	ENCTYPE_CAST5_40_OFB64,
+  { "CAST5_40_OFB64",	ENCTYPE_CAST5_40_OFB64,
+			ENCFLAG_DEFAULT,
 			castexp_ofb64_encrypt,
 			castexp_ofb64_decrypt,
 			castexp_ofb64_init,
@@ -200,8 +212,8 @@ static Encryptions encryptions[] = {
 			castexp_ofb64_session,
 			castexp_ofb64_keyid,
 			cast_ofb64_printsub },
-#endif
-    { 0, },
+#endif /* CAST_ENCRYPTION || CAST_EXPORT_ENCRYPTION */
+  { 0, },
 };
 
 static unsigned char str_send[64] = { IAC, SB, TELOPT_ENCRYPT,
@@ -258,7 +270,7 @@ encrypt_init(name, server)
 
 	Name = name;
 	Server = server;
-	i_support_encrypt = i_support_decrypt = 0;
+	/*i_support_encrypt = i_support_decrypt = 0;*/
 	remote_supports_encrypt = remote_supports_decrypt = 0;
 	encrypt_mode = 0;
 	decrypt_mode = 0;
@@ -274,11 +286,18 @@ encrypt_init(name, server)
 		if (encrypt_debug_mode)
 			printf(">>>%s: I will support %s\r\n",
 				Name, ENCTYPE_NAME(ep->type));
-		i_support_encrypt |= typemask(ep->type);
-		i_support_decrypt |= typemask(ep->type);
-		if ((i_wont_support_decrypt & typemask(ep->type)) == 0)
+		if ((i_wont_support_decrypt & typemask(ep->type)) == 0 &&
+		    (Server || (ep->flags & ENCFLAG_CLIENT_ENABLE) || (i_support_decrypt & typemask(ep->type)))
+		    ) {
 			if ((str_send[str_suplen++] = ep->type) == IAC)
 				str_send[str_suplen++] = IAC;
+			i_support_encrypt |= typemask(ep->type);
+			i_support_decrypt |= typemask(ep->type);
+		}
+		else {
+			i_wont_support_encrypt |= typemask(ep->type);
+			i_wont_support_decrypt |= typemask(ep->type);
+		}
 		if (ep->init)
 			(*ep->init)(Server);
 		++ep;
@@ -287,6 +306,16 @@ encrypt_init(name, server)
 	str_send[str_suplen++] = SE;
 }
 
+void
+encrypt_reset()
+{
+	/* Reset the state of the ENCRYPT option */
+	encrypt_mode = 0;
+	decrypt_mode = 0;
+	encrypt_output = 0;
+	decrypt_input = 0;
+}
+
 	void
 encrypt_list_types()
 {
@@ -300,15 +329,15 @@ encrypt_list_types()
 }
 
 	int
-EncryptEnable(type, mode)
+EncryptBegin(type, mode)
 	char *type, *mode;
 {
 	if (isprefix(type, "help") || isprefix(type, "?")) {
-		printf("Usage: encrypt enable <type> [input|output]\n");
+		printf("Usage: encrypt begin <type> [input|output]\n");
 		encrypt_list_types();
 		return(0);
 	}
-	if (EncryptType(type, mode))
+	if (EncryptEnable(type, mode))
 		return(EncryptStart(mode));
 	return(0);
 }
@@ -333,12 +362,14 @@ EncryptDisable(type, mode)
 			if (decrypt_mode == ep->type)
 				EncryptStopInput();
 			i_wont_support_decrypt |= typemask(ep->type);
+			i_support_decrypt &= ~typemask(ep->type);
 			ret = 1;
 		}
 		if ((mode == 0) || (isprefix(mode, "output"))) {
 			if (encrypt_mode == ep->type)
 				EncryptStopOutput();
 			i_wont_support_encrypt |= typemask(ep->type);
+			i_support_encrypt &= ~typemask(ep->type);
 			ret = 1;
 		}
 		if (ret == 0)
@@ -348,7 +379,7 @@ EncryptDisable(type, mode)
 }
 
 	int
-EncryptType(type, mode)
+EncryptEnable(type, mode)
 	char *type;
 	char *mode;
 {
@@ -356,7 +387,7 @@ EncryptType(type, mode)
 	int ret = 0;
 
 	if (isprefix(type, "help") || isprefix(type, "?")) {
-		printf("Usage: encrypt type <type> [input|output]\n");
+		printf("Usage: encrypt enable <type> [input|output]\n");
 		encrypt_list_types();
 	} else if ((ep = (Encryptions *)genget(type, encryptions,
 						sizeof(Encryptions))) == 0) {
@@ -367,11 +398,13 @@ EncryptType(type, mode)
 		if ((mode == 0) || isprefix(mode, "input")) {
 			decrypt_mode = ep->type;
 			i_wont_support_decrypt &= ~typemask(ep->type);
+			i_support_decrypt |= typemask(ep->type);
 			ret = 1;
 		}
 		if ((mode == 0) || isprefix(mode, "output")) {
 			encrypt_mode = ep->type;
 			i_wont_support_encrypt &= ~typemask(ep->type);
+			i_support_encrypt |= typemask(ep->type);
 			ret = 1;
 		}
 		if (ret == 0)
@@ -466,31 +499,31 @@ EncryptStopOutput()
 encrypt_display()
 {
 	if (encrypt_output)
-		printf("Currently encrypting output with %s\r\n",
-			ENCTYPE_NAME(encrypt_mode));
+		printf("Currently encrypting output with %s (%d bits)\r\n",
+			ENCTYPE_NAME(encrypt_mode), ENCTYPE_BITS(encrypt_mode));
 	if (decrypt_input)
-		printf("Currently decrypting input with %s\r\n",
-			ENCTYPE_NAME(decrypt_mode));
+		printf("Currently decrypting input with %s (%d bits)\r\n",
+			ENCTYPE_NAME(decrypt_mode), ENCTYPE_BITS(decrypt_mode));
 }
 
 	int
 EncryptStatus()
 {
 	if (encrypt_output)
-		printf("Currently encrypting output with %s\r\n",
-			ENCTYPE_NAME(encrypt_mode));
+		printf("Currently encrypting output with %s (%d bits)\r\n",
+			ENCTYPE_NAME(encrypt_mode), ENCTYPE_BITS(encrypt_mode));
 	else if (encrypt_mode) {
 		printf("Currently output is clear text.\r\n");
-		printf("Last encryption mode was %s\r\n",
-			ENCTYPE_NAME(encrypt_mode));
+		printf("Last encryption mode was %s (%d bits)\r\n",
+			ENCTYPE_NAME(encrypt_mode), ENCTYPE_BITS(encrypt_mode));
 	}
 	if (decrypt_input) {
-		printf("Currently decrypting input with %s\r\n",
-			ENCTYPE_NAME(decrypt_mode));
+		printf("Currently decrypting input with %s (%d bits)\r\n",
+			ENCTYPE_NAME(decrypt_mode), ENCTYPE_BITS(decrypt_mode));
 	} else if (decrypt_mode) {
 		printf("Currently input is clear text.\r\n");
-		printf("Last decryption mode was %s\r\n",
-			ENCTYPE_NAME(decrypt_mode));
+		printf("Last decryption mode was %s (%d bits)\r\n",
+			ENCTYPE_NAME(decrypt_mode), ENCTYPE_BITS(decrypt_mode));
 	}
 	return 1;
 }
@@ -498,6 +531,28 @@ EncryptStatus()
 	void
 encrypt_send_support()
 {
+    Encryptions *ep = encryptions;
+
+    str_send[0] = IAC;
+    str_send[1] = SB;
+    str_send[2] = TELOPT_ENCRYPT;
+    str_send[3] = ENCRYPT_SUPPORT;
+    str_suplen = 4;
+
+    /*
+     * Now that i_wont_support_decrypt is computed at runtime based upon
+     * the success or failure of each cipher we have to compute the send
+     * string just before sending it.
+     */
+    while (ep->type) {
+        if ((i_wont_support_decrypt & typemask(ep->type)) == 0)
+            if ((str_send[str_suplen++] = ep->type) == IAC)
+                str_send[str_suplen++] = IAC;
+        ++ep;
+    }
+    str_send[str_suplen++] = IAC;
+    str_send[str_suplen++] = SE;
+
 	if (str_suplen) {
 		/*
 		 * If the user has requested that decryption start
@@ -576,10 +631,15 @@ encrypt_support(typelist, cnt)
 
 	while (cnt-- > 0) {
 		type = *typelist++;
-		if (encrypt_debug_mode)
+            if (encrypt_debug_mode) {
+                    if (ENCRYPT_NAME_OK(type) )
 			printf(">>>%s: He is supporting %s (%d)\r\n",
 				Name,
 				ENCTYPE_NAME(type), type);
+                    else 
+                        printf(">>>%s: He is supporting UNKNOWN (%d)\r\n",
+                                Name,type);
+            }
 		if ((type < ENCTYPE_CNT) &&
 		    (I_SUPPORT_ENCRYPT & typemask(type))) {
 			remote_supports_decrypt |= typemask(type);
@@ -721,8 +781,9 @@ encrypt_start(data, cnt)
 	if (ep = finddecryption(decrypt_mode)) {
 		decrypt_input = ep->input;
 		if (encrypt_verbose)
-			printf("[ Input is now decrypted with type %s ]\r\n",
-				ENCTYPE_NAME(decrypt_mode));
+			printf("[ Input is now decrypted with type %s (%d bits) ]\r\n",
+				ENCTYPE_NAME(decrypt_mode),
+				ENCTYPE_BITS(decrypt_mode));
 		if (encrypt_debug_mode)
 			printf(">>>%s: Start to decrypt input with type %s\r\n",
 				Name, ENCTYPE_NAME(decrypt_mode));
@@ -744,11 +805,19 @@ encrypt_session_key(key, server)
 {
 	Encryptions *ep = encryptions;
 
+#ifdef TLS
+	if(tls_active)
+		return;
+#endif /* TLS */
+
 	havesessionkey = 1;
 
 	while (ep->type) {
 		if (ep->session)
-			(*ep->session)(key, server);
+                    if ((*ep->session)(key, server) < 0) {
+                        i_wont_support_encrypt |= typemask(ep->type);
+                        i_wont_support_decrypt |= typemask(ep->type);
+                    }
 #ifdef notdef
 		if (!encrypt_output && autoencrypt && !server)
 			encrypt_start_output(ep->type);
@@ -802,6 +871,7 @@ encrypt_request_start(data, cnt)
 
 static unsigned char str_keyid[(MAXKEYLEN*2)+5] = { IAC, SB, TELOPT_ENCRYPT };
 
+void
 encrypt_enc_keyid(keyid, len)
 	unsigned char *keyid;
 	int len;
@@ -809,6 +879,7 @@ encrypt_enc_keyid(keyid, len)
 	encrypt_keyid(&ki[1], keyid, len);
 }
 
+void
 encrypt_dec_keyid(keyid, len)
 	unsigned char *keyid;
 	int len;
@@ -925,7 +996,7 @@ encrypt_start_output(type)
 		}
 		return;
 	}
-	if (ep->output == encrypt_output)
+	if (ep->output == encrypt_output)	/* TJW: still need this? */
 	  return;
 	if (ep->start) {
 		i = (*ep->start)(DIR_ENCRYPT, Server);
@@ -962,8 +1033,8 @@ encrypt_start_output(type)
 		printf(">>>%s: Started to encrypt output with type %s\r\n",
 			Name, ENCTYPE_NAME(type));
 	if (encrypt_verbose)
-		printf("[ Output is now encrypted with type %s ]\r\n",
-			ENCTYPE_NAME(type));
+		printf("[ Output is now encrypted with type %s (%d bits) ]\r\n",
+			ENCTYPE_NAME(type), ENCTYPE_BITS(type));
 }
 
 	void
@@ -1031,12 +1102,25 @@ encrypt_wait()
 			return;
 }
 
+void
+decrypt_wait()
+{
+	register int encrypt, decrypt;
+	if (encrypt_debug_mode)
+		printf(">>>%s: in decrypt_wait\r\n", Name);
+	if (!havesessionkey || !(I_SUPPORT_DECRYPT & remote_supports_encrypt))
+		return;
+	while (autoencrypt && !decrypt_input)
+		if (telnet_spin())
+			return;
+}
+
         int
 encrypt_is_encrypting()
 {
-    if ( encrypt_output && decrypt_input )
-        return(1);
-    return(0);
+    if (encrypt_output && decrypt_input)
+        return 1;
+    return 0;
 }
 
 	void
