@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-2001  The Stanford SRP Authentication Project
+ * Copyright (c) 1997-2003  The Stanford SRP Authentication Project
  * All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
@@ -62,7 +62,6 @@
 #include "auth.h"
 #include "misc.h"
 
-#include "t_pwd.h"
 #include "srp.h"
 
 extern auth_debug_mode;
@@ -214,10 +213,11 @@ srp_is(ap, data, cnt)
 	unsigned char *data;
 	int cnt;
 {
-  char pbuf[2 * MAXPARAMLEN + 5];
+  char * pbuf;
   char * ptr;
-  char hexbuf[MAXHEXPARAMLEN];
+  /**
   struct t_passwd * pass;
+  */
   cstr * resp;
 /*
   FILE * passfp = NULL;
@@ -229,6 +229,7 @@ srp_is(ap, data, cnt)
 #ifdef  TLS
   int flen;
 #endif
+  int modlen, genlen;
   unsigned char type_check[2];
 
   if(cnt-- < 1)
@@ -275,7 +276,9 @@ srp_is(ap, data, cnt)
       Data(ap, SRP_REJECT, (void *) "SRP server init failed", -1);
       break;
     }
+    /**
     pass = gettpnam(UserNameRequested);
+    */
 /*
     t_closepw(tpw);
     if(passfp)
@@ -287,6 +290,14 @@ srp_is(ap, data, cnt)
     tconf = NULL;
 */
 
+    SRP_set_server_lookup(s_srp, SRP_SERVER_system_lookup());
+    if(SRP_set_username(s_srp, UserNameRequested) != SRP_SUCCESS) {
+      if (auth_debug_mode)
+	printf("User %s not found\r\n", UserNameRequested);
+      Data(ap, SRP_REJECT, (void *) "Password not set", -1);
+      break;
+    }
+    /*
     if(pass == NULL) {
       if (auth_debug_mode)
 	printf("User %s not found\r\n", UserNameRequested);
@@ -304,25 +315,30 @@ srp_is(ap, data, cnt)
       Data(ap, SRP_REJECT, (void *) "SRP parameter init failed", -1);
       break;
     }
+    */
 
+    modlen = BigIntegerByteLen(s_srp->modulus);
+    genlen = BigIntegerByteLen(s_srp->generator);
+    pbuf = (char *)malloc(modlen + genlen + s_srp->salt->length + 7);
     ptr = pbuf;
 
-    encode_length(ptr, pass->tc.modulus.len);
+    encode_length(ptr, modlen);
     ptr += 2;
-    memcpy(ptr, pass->tc.modulus.data, pass->tc.modulus.len);
-    ptr += pass->tc.modulus.len;
+    BigIntegerToBytes(s_srp->modulus, ptr, modlen);
+    ptr += modlen;
 
-    encode_length(ptr, pass->tc.generator.len);
+    encode_length(ptr, genlen);
     ptr += 2;
-    memcpy(ptr, pass->tc.generator.data, pass->tc.generator.len);
-    ptr += pass->tc.generator.len;
+    BigIntegerToBytes(s_srp->generator, ptr, genlen);
+    ptr += genlen;
 
-    encode_length(ptr, pass->tp.salt.len);
+    encode_length(ptr, s_srp->salt->length);
     ptr += 2;
-    memcpy(ptr, pass->tp.salt.data, pass->tp.salt.len);
-    ptr += pass->tp.salt.len;
+    memcpy(ptr, s_srp->salt->data, s_srp->salt->length);
+    ptr += s_srp->salt->length;
 
     Data(ap, SRP_PARAMS, pbuf, ptr - pbuf);  /* Ideally, this is flushed */
+    free(pbuf);
 
     if(SRP_gen_pub(s_srp, &B) != SRP_SUCCESS) {
       if (auth_debug_mode)
@@ -386,7 +402,7 @@ srp_is(ap, data, cnt)
       break;
     }
 
-    if(cnt < RESPONSE_LEN) {
+    if(cnt < RFC2945_RESP_LEN) {
       if (auth_debug_mode)
 	printf("Protocol error: malformed response\r\n");
       Data(ap, SRP_REJECT, (void *) "Protocol error: malformed response", -1);
@@ -436,14 +452,16 @@ srp_reply(ap, data, cnt)
 	unsigned char *data;
 	int cnt;
 {
-  struct t_num n;
-  struct t_num g;
-  struct t_num s;
+  unsigned char * ndata;
+  int nlen;
+  unsigned char * gdata;
+  int glen;
+  unsigned char * sdata;
+  int slen;
 
   cstr * resp;
   cstr * A;
 
-  char hexbuf[MAXHEXPARAMLEN];
   int pflag;
 #ifdef	ENCRYPTION
   Session_Key skey;
@@ -465,7 +483,7 @@ srp_reply(ap, data, cnt)
     auth_send_retry();
     break;
   case SRP_ACCEPT:
-    if(c_srp == NULL || cnt < RESPONSE_LEN || !waitresponse) {
+    if(c_srp == NULL || cnt < RFC2945_RESP_LEN || !waitresponse) {
       if (auth_debug_mode)
 	printf("Protocol error\r\n");
       break;
@@ -507,48 +525,48 @@ srp_reply(ap, data, cnt)
       break;
     }
 
-    n.len = decode_length(data);
+    nlen = decode_length(data);
     data += 2;
     cnt -= 2;
-    if(n.len > cnt) {
+    if(nlen > cnt) {
       if (auth_debug_mode)
 	printf("n too long\r\n");
       break;
     }
-    n.data = data;
-    data += n.len;
-    cnt -= n.len;
+    ndata = data;
+    data += nlen;
+    cnt -= nlen;
 
-    printf("[ Using %d-bit modulus for '%s' ]\r\n", 8 * n.len, UserNameRequested);
+    printf("[ Using %d-bit modulus for '%s' ]\r\n", 8 * nlen, UserNameRequested);
 
-    g.len = decode_length(data);
+    glen = decode_length(data);
     data += 2;
     cnt -= 2;
-    if(g.len > cnt) {
+    if(glen > cnt) {
       if (auth_debug_mode)
 	printf("g too long\r\n");
       break;
     }
-    g.data = data;
-    data += g.len;
-    cnt -= g.len;
+    gdata = data;
+    data += glen;
+    cnt -= glen;
 
-    s.len = decode_length(data);
+    slen = decode_length(data);
     data += 2;
     cnt -= 2;
-    if(s.len > cnt) {
+    if(slen > cnt) {
       if (auth_debug_mode)
 	printf("salt too long\r\n");
       break;
     }
-    s.data = data;
-    data += s.len;
-    cnt -= s.len;
+    sdata = data;
+    data += slen;
+    cnt -= slen;
 
     c_srp = SRP_new(SRP_RFC2945_client_method());
     if(c_srp == NULL ||
        SRP_set_username(c_srp, UserNameRequested) != SRP_SUCCESS ||
-       SRP_set_params(c_srp, n.data, n.len, g.data, g.len, s.data, s.len) !=
+       SRP_set_params(c_srp, ndata, nlen, gdata, glen, sdata, slen) !=
        SRP_SUCCESS) {
       printf("[ Parameter initialization error ]\r\n");
       auth_send_retry();
