@@ -1473,11 +1473,6 @@ startslave(host, autologin, autoname)
 {
 	register int i;
 	int syncpipe[2];
-#if defined(OSF) || defined(ultrix) || defined(__hpux)
-	time_t time();
-#else
-	long time();
-#endif
 	char name[256];
 #ifdef	NEWINIT
 	extern char *ptyip;
